import logging
import logging.handlers
import os


# 200kb * 5 files = 1mb of logs
DEFAULT_LOG_MAX_BYTES = 200000  # 200kb
DEFAULT_LOG_BACKUP_COUNT = 5

DEFAULT_LOG_LEVEL = 'INFO'

DEFAULT_CONSOLE_FORMATTER = '%(message)s'
DEFAULT_FILE_FORMATTER = '%(asctime)s - %(levelname)s - %(module)s.%(funcName)s - %(message)s'


class Logger:
    def __init__(self, name: str, level: str = DEFAULT_LOG_LEVEL):
        """Setup a logger based on a provided set of input.

        - `name`: Each module that requires logging should instantiate a new class and pass a
        new name based on the module using logging. Typically for most purposes, this
        should be `__name__` passed as `name` or the root name of your package.
        - `level`: Every logger needs a level. Logged messages of a greater or equal value
        to the log level will be logged while those of a lesser value will be ignored.
        - `_logger`: This is the actual `logging.Logger` object wrapped on `woodchips.Logger`. This
        is not intended to be used externally, instead use `woodchips.get()` to call methods such
        as `.info()`, `.warning()` etc.
        """
        # Parameter variables
        self.name = name
        self.level = level

        # Internal variables
        self._logger = get(self.name)
        log_level = self._validate_log_level()

        self._logger.setLevel(log_level)

    def log_to_console(self, formatter: str = DEFAULT_CONSOLE_FORMATTER) -> None:
        """Adds a console handler to a logger."""
        console_handler = logging.StreamHandler()

        console_formatter = logging.Formatter(formatter)
        console_handler.setFormatter(console_formatter)
        self._logger.addHandler(console_handler)

    def log_to_file(
        self,
        location: str,
        formatter: str = DEFAULT_FILE_FORMATTER,
        log_size: int = DEFAULT_LOG_MAX_BYTES,
        num_of_logs: int = DEFAULT_LOG_BACKUP_COUNT,
    ) -> None:
        """Adds a file handler to a logger."""
        if not os.path.exists(location):
            os.makedirs(location)

        # Splitting on the period assumes the user specified `__name__` so we can get
        # the root package name for log filenames, otherwise we'll just use the name.
        log_name = self._logger.name.split('.')[0] + '.log'
        log_file = os.path.join(location, log_name)

        file_handler = logging.handlers.RotatingFileHandler(
            log_file,
            maxBytes=log_size,
            backupCount=num_of_logs,
        )

        file_formatter = logging.Formatter(formatter)
        file_handler.setFormatter(file_formatter)
        self._logger.addHandler(file_handler)

    def _validate_log_level(self) -> int:
        """Internal utility to validate the input log level is valid, raise an error if not."""
        log_levels = {
            'CRITICAL': logging.CRITICAL,  # 50
            'ERROR': logging.ERROR,  # 40
            'WARNING': logging.WARNING,  # 30
            'INFO': logging.INFO,  # 20
            'DEBUG': logging.DEBUG,  # 10
            'NOTSET': logging.NOTSET,  # 0
        }

        try:
            log_level = log_levels[self.level.upper()]
        except KeyError as error:
            raise KeyError(
                f'Could not setup Woodchips due to invalid log level: {error}, must be one of {log_levels.keys()}'  # noqa
            )

        return log_level


def get(logger_name: str) -> logging.Logger:
    """Gets or creates a logger instance by name.

    If no logger exists with the specified name, a new logger object
    will be created with that name.
    """
    logger = logging.getLogger(logger_name)

    return logger



import inspect
import logging
import os
import tempfile
from unittest.mock import (
    mock_open,
    patch,
)

import pytest

import woodchips


MOCK_LOG_PATH = os.path.join('test', 'mock-dir')
LOG_PATH_EXISTS = os.path.join('test', 'logs')
LOG_LEVEL = 'INFO'


@patch('os.makedirs')
def test_logger_class(mock_make_dirs):
    """Test setting up the Woodchips `Logger` class works correctly."""
    function_name = inspect.stack()[0][3]

    my_logger = woodchips.Logger(
        name=function_name,
    )

    assert my_logger._logger.name == function_name
    assert logging.getLevelName(my_logger._logger.getEffectiveLevel()) == 'INFO'


def test_log_to_console():
    """Test that the console handler gets setup correctly."""
    function_name = inspect.stack()[0][3]

    my_logger = woodchips.Logger(name=function_name)
    my_logger.log_to_console()

    assert type(my_logger._logger.handlers[0]) == logging.StreamHandler
    assert my_logger._logger.handlers[0].formatter._fmt == '%(message)s'


def test_log_to_console_with_formatter():
    """Test that the console handler gets setup correctly with a formatter."""
    function_name = inspect.stack()[0][3]

    my_logger = woodchips.Logger(name=function_name)
    my_logger.log_to_console(formatter='%(asctime)s')

    assert type(my_logger._logger.handlers[0]) == logging.StreamHandler
    assert my_logger._logger.handlers[0].formatter._fmt == '%(asctime)s'


@pytest.mark.parametrize(
    'log_level, expected_log_value',
    [
        ('CRITICAL', 50),
        ('ERROR', 40),
        ('WARNING', 30),
        ('INFO', 20),
        ('DEBUG', 10),
        ('notset', 0),  # lowercased to ensure we uppercase all user input for this var
    ],
)
def test_validate_log_level(log_level, expected_log_value):
    """Tests that we map log levels correctly."""
    function_name = inspect.stack()[0][3]

    my_logger = woodchips.Logger(name=function_name, level=log_level)
    active_log_level = my_logger._validate_log_level()

    assert active_log_level == expected_log_value


def test_validate_log_level_invalid_level():
    """Test that we raise an error when a bad log level is passed in."""
    function_name = inspect.stack()[0][3]

    with pytest.raises(KeyError) as error:
        my_logger = woodchips.Logger(name=function_name, level='BAD_LOG_LEVEL')
        my_logger._validate_log_level()

    assert (
        str(error.value)
        == '"Could not setup Woodchips due to invalid log level: \'BAD_LOG_LEVEL\', must be one of'
        ' dict_keys([\'CRITICAL\','
        ' \'ERROR\', \'WARNING\', \'INFO\', \'DEBUG\', \'NOTSET\'])"'
    )


def test_log_to_file():
    """Tests that the file handler gets setup correctly."""
    function_name = inspect.stack()[0][3]

    with tempfile.TemporaryDirectory() as temp_dir:
        my_logger = woodchips.Logger(name=function_name)
        my_logger.log_to_file(
            location=temp_dir,
            formatter='%(asctime)s',
            log_size=10000,
            num_of_logs=2,
        )

        assert type(my_logger._logger.handlers[0]) == logging.handlers.RotatingFileHandler
        assert my_logger._logger.handlers[0].formatter._fmt == '%(asctime)s'
        assert function_name in my_logger._logger.handlers[0].baseFilename
        assert my_logger._logger.handlers[0].maxBytes == 10000
        assert my_logger._logger.handlers[0].backupCount == 2
        assert os.path.exists(temp_dir)


@patch('os.makedirs')
def test_log_to_file_dir_doesnt_exist(mock_makedirs):
    """Tests that the file handler gets setup correctly when no `location` exists already."""
    function_name = inspect.stack()[0][3]

    with patch('builtins.open', mock_open()):
        my_logger = woodchips.Logger(name=function_name)
        my_logger.log_to_file(location='mock_dir')

    mock_makedirs.assert_called_once()


def test_get_logger():
    """Tests that we `get` a logger instance by name."""
    custom_logger_name = 'custom_logger_name'

    my_logger = woodchips.Logger(
        name=custom_logger_name,
    )

    retrieved_logger = woodchips.get(custom_logger_name)

    assert retrieved_logger == my_logger._logger
    assert retrieved_logger.name == my_logger.name == custom_logger_name



import setuptools


with open('README.md', 'r') as fh:
    long_description = fh.read()

DEV_REQUIREMENTS = [
    'bandit == 1.7.*',
    'black == 23.*',
    'build == 0.10.*',
    'flake8 == 6.*',
    'isort == 5.*',
    'mypy == 1.3.*',
    'pytest == 7.*',
    'pytest-cov == 4.*',
    'twine == 4.*',
]

setuptools.setup(
    name='woodchips',
    version='1.0.0',
    description='The cutest little logger you\'ve ever seen.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='http://github.com/justintime50/woodchips',
    author='justintime50',
    license='MIT',
    packages=setuptools.find_packages(
        exclude=[
            'examples',
            'test',
        ]
    ),
    package_data={
        'woodchips': [
            'py.typed',
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    extras_require={
        'dev': DEV_REQUIREMENTS,
    },
    python_requires='>=3.8, <4',
)
